//
// C++ Implementation: chiquadrat
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "chiquadrat.h"
#include "diagram.h"
#include "balkendiagram.h"

#include <QDoubleSpinBox>
#include <QLabel>
#include <QPushButton>
#include <QLayout>
#include <QStandardItemModel>
#include <QtDebug>
#include <cmath>

ChiQuadrat::ChiQuadrat(QWidget* parent): ModelClient(parent),td(0),N(10),input_filter(""),filter_spalte(-1)
{
	l = new QGridLayout(this);
	sX0 = new QDoubleSpinBox(this);
	sX0->setSingleStep(0.01);
	sX0->setDecimals(5);
	
	sSigma = new QDoubleSpinBox(this);
	sSigma->setSingleStep(0.01);
	sSigma->setDecimals(5);
	sSigma->setValue(0);
	
	sAlpha = new QDoubleSpinBox(this);
	sAlpha->setSingleStep(0.01);
	sAlpha->setDecimals(5);
	sAlpha->setMinimum(0.00001);
	sAlpha->setMaximum(0.99999);
	sAlpha->setValue(0.05);
	sAlpha->setEnabled(false);
	
	sN = new QDoubleSpinBox(this);
	sN->setMinimum(6);
	sN->setMaximum(30);
	sN->setValue(N);
	
	QPushButton *pTest = new QPushButton(tr("Testen"),this);
	
	lChi = new QLabel(tr("0.0"),this);
	QFont fn = lChi->font();
	fn.setBold(true);
	lChi->setFont(fn);
	
	QLabel *l0;
	l->addWidget(l0=new QLabel(QChar(0x3bd),this),0,1);
	l0->setAlignment(Qt::AlignRight);
	l->addWidget(sX0,0,2);
	l->addWidget(l0=new QLabel(QChar(0x3C3),this),0,3);
	l0->setAlignment(Qt::AlignRight);
	l->addWidget(sSigma,0,4);
	l->addWidget(pTest,0,6);
	l->addWidget(l0=new QLabel(QChar(0x3B1),this),1,1);
	l0->setAlignment(Qt::AlignRight);
	l->addWidget(sAlpha,1,2);
	l->addWidget(l0=new QLabel(tr("m"),this),1,3);
	l0->setAlignment(Qt::AlignRight);
	l->addWidget(sN,1,4);
	l->addWidget(l0=new QLabel(QString()+QChar(0x3C7)+QChar(178)+"=",this),1,5);
	l0->setAlignment(Qt::AlignRight);
	l0->setFont(fn);
	l->addWidget(lChi,1,6);
	
	l->addWidget(td=new BalkenDiagram(this),2,0,8,8);
	
	connect(pTest,SIGNAL(clicked()),this,SLOT(updateDiagram()));
}


ChiQuadrat::~ChiQuadrat()
{
}


void ChiQuadrat::loadData(QDataStream& s, int ver)
{
    ModelClient::loadData(s, ver);
}

void ChiQuadrat::saveData(QDataStream& s, int ver)
{
    ModelClient::saveData(s, ver);
}

void ChiQuadrat::setModel(QStandardItemModel* m)
{
	if(_model) disconnect(_model,SIGNAL(dataChanged( const QModelIndex &, const QModelIndex & )),this,SLOT(updateDiagram()));
	_model = m;
	connect(_model,SIGNAL(dataChanged( const QModelIndex &, const QModelIndex &)),this,SLOT(updateDiagram()));
	updateDiagram();
}

void ChiQuadrat::setSpalten(const QVector<int>& sp)
{
	Spalten = sp;
	updateDiagram();
}

void ChiQuadrat::updateDiagram()
{
	if(!td || !_model || Spalten.count()!=1 ) return;
	using namespace std;
	
	const int model_column = Spalten[0];
	
	
	// Einstellungen einlesen
	nu = sX0->value();
	si = sSigma->value();
	N = sN->value();
	
	// Werte aus Model einlesen
	QVector<double> werte;
	for(int i=0;i<_model->rowCount();++i)
	{
		if( filter_spalte>=0 && filter_wert!=_model->data(_model->index(i,filter_spalte)).toString()) continue;
		if( !input_filter.isNull() && input_filter == _model->data(_model->index(i,model_column))) continue;
		werte << _model->data(_model->index(i,model_column)).toDouble();
	}
	
	// Grenzen und Interval
	const double min = *min_element(werte.begin(),werte.end());
	const double max = *max_element(werte.begin(),werte.end());
	const double interval = (max-min)/N;
	
	sX0->setMinimum(min);
	sX0->setMaximum(max);
	//sX0->setValue(min+(max-min)/2.0);
	sSigma->setMaximum(werte.count()*5.0);
	sSigma->setMinimum(0.00001);
	//sSigma->setValue(werte.count());
	
	QVector<double> h(N); // Haeufigkeitsverteilung
	QStringList labels;
	int count=0;
	for(double i=min;i<=max;i+=interval,++count)
	{
		labels << QString::number(i+interval/2.0);
		foreach(double v,werte)
		{
			if(i<=v && v<i+interval) h[count]++;
		}
	}
	
	// Chi Quadrat
	
	const double m = h.count();
	const double n = werte.count();
	
	double chi = 0.0;
	QVector<double> No;
	for(int j=0;j<m;++j)
	{
		const double no= n*w(min+(j+1)*interval,min+j*interval);
		No<<no;
		chi += pow(h[j]-no,2)/no;
	}
	
	lChi->setText(QString::number(chi));
	
	QVector<double> dd;
	dd += h;
	dd += No;
	
	QStringList colabels;
	colabels<<"Daten";
	colabels<<"Normal";
	
	// Diagramm updaten
	td->setLabels(labels);
	td->setColLabels(colabels);
	td->setRowWidth(N);
	td->setData(dd);
}

double ChiQuadrat::w(double x,double x0)
{
	double s = 0.0;
	for(double i=x0;i<x;i+=0.0001)
	{
		s += exp(-0.5*pow(i-nu,2)/pow(si,2))/(pow(si,2)*sqrt(2.0*3.141592));
	}
	return s;
}
