//
// C++ Implementation: Diagram
//
// Description: Diagramm Basisklasse
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "diagram.h"
#include <QDrag>
#include <QMimeData>
#include <QApplication>
#include <QAction>
#include <QMessageBox>
#include <QColorDialog>
#include <QFontDialog>
#include <QInputDialog>

Diagram::Diagram(QWidget *parent)
	:QWidget(parent),row_width(0)
{
	setMinimumSize(100,140);
	colors.append(QColor(Qt::red));
	colors.append(QColor(Qt::blue));
	colors.append(QColor(Qt::cyan));
	colors.append(QColor(Qt::magenta));
	colors.append(QColor(Qt::yellow));
	colors.append(QColor(Qt::green));
	colors.append(QColor(Qt::gray));
	colors.append(QColor(Qt::darkBlue));
	colors.append(QColor(Qt::darkCyan));
	colors.append(QColor(Qt::darkMagenta));
	QAction *a = new QAction(tr("Farben"),this);
	connect(a,SIGNAL(triggered()),this,SLOT(colorAction()));
	actions << a;
	a = new QAction(tr("Schriftart"),this);
	connect(a,SIGNAL(triggered()),this,SLOT(fontAction()));
	actions << a;
}

void Diagram::setData(const QVector<double> &d)
{
	data = d;
	update();
}

void Diagram::setLabels(const QStringList &l)
{
	labels = l;
	update();
}

void Diagram::setColLabels(const QStringList &l)
{
	colLabels = l;
	update();
}

void Diagram::setRowWidth(int w)
{
	row_width = w;
	update();
}

void Diagram::setTitle(const QString &t)
{
	_title = t;
	update();
}

void Diagram::setValuePostfix(const QString& p)
{
	value_postfix = p;
}

// Drag and Drop
 void Diagram::mousePressEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton)
         dragStartPosition = event->pos();
 }


 void Diagram::mouseMoveEvent(QMouseEvent *event)
 {
     if (!(event->buttons() & Qt::LeftButton))
         return;
     if ((event->pos() - dragStartPosition).manhattanLength()
          < QApplication::startDragDistance())
         return;

     QDrag *drag = new QDrag(this);
     QMimeData *mimeData = new QMimeData;
     // Get Image
     QPixmap px = QPixmap::grabWidget(this);
     mimeData->setImageData(px.toImage());
     //mimeData->setData(mimeType, data);
     drag->setMimeData(mimeData);

     Qt::DropAction dropAction = drag->start(Qt::CopyAction | Qt::MoveAction);
 }

 QVector<QAction*> Diagram::actionList()
 {
	 return actions;
 }
 
void Diagram::colorAction()
{
	int idx=0;
	if(!labels.isEmpty())
	{
		bool ok;
		QString l = QInputDialog::getItem(this,tr("Zeile"),tr("Zeile auswhlen deren Farbe sie ndern wollen"),labels,0,&ok);
		if(!ok) return;
		idx = labels.indexOf(l);
	}
	QColor oc = colors.at(idx%colors.count());
	QColor c = QColorDialog::getColor(oc,this);
	colors[idx%colors.count()]=c;
	update();
}

void Diagram::fontAction()
{
	bool ok;
	QFont f = QFontDialog::getFont(&ok,font(),this);
	if(ok) setFont(f);
	update();
}

void Diagram::loadData(QDataStream &s,int ver)
{
	QFont f;
	s >> f; // Schriftart
	s >> colors;
}

void Diagram::saveData(QDataStream &s,int ver)
{
	s << font();
	s << colors;
}
