//
// C++ Implementation: TortenDiagram
//
// Description:
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "tortendiagram.h"

#include <QPainter>
#include <QtDebug>
#include <numeric>
#include <algorithm>
#include <math.h>

TortenDiagram::TortenDiagram(QWidget *parent)
	:Diagram(parent)
{
	setMinimumSize(100,140);
	colors.append(QColor(Qt::red));
	colors.append(QColor(Qt::blue));
	colors.append(QColor(Qt::cyan));
	colors.append(QColor(Qt::magenta));
	colors.append(QColor(Qt::yellow));
	colors.append(QColor(Qt::green));
	colors.append(QColor(Qt::gray));
	colors.append(QColor(Qt::darkBlue));
	colors.append(QColor(Qt::darkCyan));
	colors.append(QColor(Qt::darkMagenta));
}

void TortenDiagram::paintEvent( QPaintEvent *)
{
	using namespace std;

	QPainter p(this);

	p.fillRect(rect(),Qt::white);
	double sum=0;
	if(row_width>0)	sum = accumulate(data.begin(),data.begin()+row_width,0);
	else sum = accumulate(data.begin(),data.end(),0);
	if(sum==0) return;
	
	// Titel
	QFont fh(p.font()),fo(p.font());
	fh.setBold(true);
	fh.setPointSize(fh.pointSize()+1);
	
	int dheight=height()-10,dwidth=width()-10;
	
	p.setFont(fh);
	if(p.fontMetrics().width(_title)>dwidth) 
		p.drawText( 5,p.fontMetrics().height(),_title);
	else
		p.drawText( (dwidth-p.fontMetrics().width(_title))/2,p.fontMetrics().height(),_title);
	dheight -= p.fontMetrics().height();
	int ly = 5+p.fontMetrics().height();
	p.setFont(fo);
	
	// Labelbreite
	QFontMetrics lfm(p.font());	
	int maxll=20;
	int maxlh=lfm.height()*labels.count()+10;
	for(int i=0;i<labels.count();++i) maxll = qMax(maxll,lfm.width(labels[i]));
	dwidth -= maxll+10;
	
	int lx = width()-maxll-10;
	
	if(labels.count()>0) p.drawRect(lx-2,ly,maxll+5,maxlh);
	
	// Kreis
	qreal r = qMin(dheight,dwidth);
	const qreal r0 = r;
	
	QRectF rec(5,height()-dheight-5,r,r);

	int i=0,start_winkel=0,y=ly+lfm.height();
	bool fround=true;
	
	for(QVector<double>::iterator it = data.begin();it!=data.end();++it)
	{
		double v = *it;
		if(v==0) continue;
		double b = (double)v/(double)sum*360.0*16.0;

		p.setBrush(colors[i%colors.count()]);
		p.setPen(Qt::NoPen);
		p.drawPie(rec,start_winkel,(int)b);
		start_winkel+=(int)b;
		
		if(fround)
		{
			// Beschriftung
			QString l;
			if(i<labels.count()) l = labels[i]; else l = QString::number(i);
			p.setPen(colors[i%colors.count()]);
			p.drawText(lx,y,l);
			y+=p.fontMetrics().height();
		}
		p.setPen(Qt::black);
		
		// Wert einzeichnen
		const double tx = 5 + r/2 + r*3.0/8.0 * cos(-(start_winkel-b/2)/16.0/180.0*3.1415);
		const double ty = 15 + r/2 + r*3.0/8.0 * sin(-(start_winkel-b/2)/16.0/180.0*3.1415);
			
		QString tt = QString::number(data[i])+value_postfix;
		p.drawText(tx-p.fontMetrics().width(tt)/2,ty+p.fontMetrics().height(),tt);
		
		// Spalte zuende
		if(row_width>0 && i>=row_width) 
		{
			sum = accumulate(it,it+row_width,0);
			start_winkel = 0;
			qreal rd = r*0.125;
			r *= 0.75;
			qreal x = rec.x()+rd;
			qreal y = rec.y()+rd;
			rec.setX(x);
			rec.setY(y);
			rec.setWidth(r);
			rec.setHeight(r);
			i=-1;
			fround=false;
		}
		++i;
	}
	
	
}
