//
// C++ Implementation: xyplotter
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "xyplotter.h"

#include <QPainter>
#include <QVector>
#include <QStandardItemModel>
#include <QtDebug>

#include <numeric>
#include <algorithm>

XYPlotter::XYPlotter(QWidget* parent): ModelClient(parent),minX(0),minY(0),maxX(0),maxY(0)
{
	
}


XYPlotter::~XYPlotter()
{
}

void XYPlotter::setModel(QStandardItemModel *m)
{
	if(_model) _model->disconnect(this,SLOT(updateData()));
	_model = m;
	connect(m,SIGNAL(dataChanged( const QModelIndex&, const QModelIndex& )),this,SLOT(updateData()));
	updateData();
}

void XYPlotter::setSpalten(int x,int y)
{
	SpalteY=y;
	SpalteX=x;
	updateData();
}

void XYPlotter::updateData()
{
	if(!_model) return;
	
	// Alte Daten loeschen
	sx.clear();
	sy.clear();
	// Neue Daten einlesen
	for(int i=0;i<_model->rowCount();++i)
	{
		double cx = _model->data(_model->index(SpalteX,i)).toDouble();
		double cy = _model->data(_model->index(SpalteY,i)).toDouble();
		sx << cx;
		sy << cy;
	}
	minX = *std::min_element(sx.begin(),sx.end());
	minY = *std::min_element(sy.begin(),sy.end());
	maxX = *std::max_element(sx.begin(),sx.end());
	maxY = *std::max_element(sy.begin(),sy.end());
	update();
}

void XYPlotter::paintEvent( QPaintEvent *ev)
{
	const double wx = maxX-minX+20;
	const double wy = maxY-minY+20;
	
	QPainter p(this);
	
	scX = width()/wx;
	scY = height()/wy;
	
	QRect ro(0,0,wx*scX,wy*scY);
	
	p.fillRect(ro,Qt::white);
	// Achsen
	p.drawLine(15,5,15,height()-15);
	p.drawLine(15,height()-15,width()-5,height()-15);
	p.drawLine(15,5,10,10);p.drawLine(15,5,20,10);
	p.drawLine(width()-5,height()-15,width()-10,height()-10);p.drawLine(width()-5,height()-15,width()-10,height()-20);
	
	p.setPen(Qt::blue);
	for(int i=0;i<sx.count();++i)
	{
		const double cx = 15+sx[i]*scX,cy= height()-sy[i]*scY-15;
		p.drawLine(cx-2,cy,cx+2,cy);
		p.drawLine(cx,cy-2,cx,cy+2);
	}
	
	paint(p); // Zusaetze
}

void XYPlotter::paint( QPainter &p)
{
	
}
